.tooltip.svelte-gvaj6l{position:absolute}.tooltip-inner.svelte-gvaj6l{position:relative;left:-50%;padding:2px 4px;font-size:12px;color:#fff;background-color:#020c1a;border-radius:4px}.tooltip-inner.svelte-gvaj6l:after{position:absolute;bottom:-3px;left:50%;width:6px;height:6px;background-color:#020c1a;content:" ";transform:rotate(45deg);transform-origin:center center}.atom-model-legend-container.svelte-gvaj6l{position:absolute;top:12px;right:12px;display:flex;max-width:200px;flex-wrap:wrap;gap:8px 16px}.atom-model-legend-item.svelte-gvaj6l{display:flex;align-items:center;color:#000}.atom-model-legend-item-ball.svelte-gvaj6l{margin-right:8px;width:8px;height:8px;border-radius:50%}.material-viewer-container.svelte-gvaj6l{width:100%;height:100%;position:relative;min-height:240px}.material-viewer-toolbar.svelte-gvaj6l{position:absolute;right:12px;top:50%;transform:translateY(-50%);display:flex;flex-direction:column;color:#000;background:#fff;box-shadow:0 6px 10px #b7c0e71a,0 8px 12px 1px #aab5df0d;border-radius:4px;padding:4px;margin-bottom:8px}.material-viewer-toolbar-btn.svelte-gvaj6l{cursor:pointer;font-size:16px;height:16px;width:16px;margin-bottom:4px}.material-viewer-toolbar-btn.svelte-gvaj6l:hover{cursor:pointer;color:#555878}.material-viewer-index-container.svelte-1d67499{width:100%;height:100%;position:relative;min-height:240px}
